@@ -13,12 +13,14 @@
 from keystoneauth1.exceptions import base
 
 
-__all__ = ('ConnectionError',
-           'ConnectTimeout',
-           'ConnectFailure',
-           'SSLError',
-           'RetriableConnectionFailure',
-           'UnknownConnectionError')
+__all__ = (
+    'ConnectionError',
+    'ConnectTimeout',
+    'ConnectFailure',
+    'SSLError',
+    'RetriableConnectionFailure',
+    'UnknownConnectionError',
+)
 
 
 class RetriableConnectionFailure(Exception):
@@ -46,6 +48,6 @@ class SSLError(ConnectionError):
 class UnknownConnectionError(ConnectionError):
     """An error was encountered but we don't know what it is."""
 
-    def __init__(self, msg, original):
-        super(UnknownConnectionError, self).__init__(msg)
+    def __init__(self, msg: str, original: Exception):
+        super().__init__(msg)
         self.original = original
