@@ -16,9 +16,9 @@ __all__ = ('ServiceProviderNotFound',)
 
 
 class ServiceProviderNotFound(base.ClientException):
-    """A Service Provider cannot be found"""
+    """A Service Provider cannot be found."""
 
-    def __init__(self, sp_id):
+    def __init__(self, sp_id: str):
         self.sp_id = sp_id
-        msg = 'The Service Provider %(sp)s could not be found' % {'sp': sp_id}
-        super(ServiceProviderNotFound, self).__init__(msg)
+        msg = f'The Service Provider {sp_id} could not be found'
+        super().__init__(msg)
